# -*- coding: utf-8 -*-
"""
SNMP v1, 2c trap daemon

SNMP v1
http://www.ietf.org/rfc/rfc1157.txt

   The form of the Trap-PDU is:

     Trap-PDU ::=
         [4]

              IMPLICIT SEQUENCE {
                 enterprise          -- type of object generating
                                     -- trap, see sysObjectID in [5]
                     OBJECT IDENTIFIER,

                 agent-addr          -- address of object generating
                     NetworkAddress, -- trap

                 generic-trap        -- generic trap type
                     INTEGER {
                         coldStart(0),
                         warmStart(1),
                         linkDown(2),
                         linkUp(3),
                         authenticationFailure(4),
                         egpNeighborLoss(5),
                         enterpriseSpecific(6)
                     },

                 specific-trap     -- specific code, present even
                     INTEGER,      -- if generic-trap is not
                                   -- enterpriseSpecific

                 time-stamp        -- time elapsed between the last
                   TimeTicks,      -- (re)initialization of the network
                                   -- entity and the generation of the
                                      trap

                 variable-bindings   -- "interesting" information
                      VarBindList
             }
"""

# 	Copyright (c) 2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



from command import command
from cxutil import ip,flags,cmask
from cxutil.utils import opts
from cxutil.exceptions import Dump

from socket import socket,AF_INET,SOCK_DGRAM
from pysnmp.proto import api
from pyasn1.codec.ber import decoder

snmp1traps = {
	0: "coldStart",
	1: "warmStart",
	2: "linkDown",
	3: "linkUp",
	4: "authenticationFailure",
	5: "egpNeighborLoss",
	6: "enterpriseSpecific",
}

class snmp_socket(object):
	"""
	SNMP socket wrapper
	"""
	sock = None
	fd = None
	signal = None

	def __init__(self,signal,bind,port):
		self.signal = signal
		self.sock = socket(AF_INET,SOCK_DGRAM)
		self.sock.bind((bind,port))
		self.fd = self.sock.fileno()

	def read(self):
		s = self.sock.recvfrom(65535)
		b = {}
		msg = s[0]
		b["src"] = s[1][0]
		b["sport"] = s[1][1]

		version = int(api.decodeMessageVersion(msg))

		if version == 0:
			b["version"] = "1"
			proto = api.protoModules[version]
			msg, x = decoder.decode(msg, asn1Spec=proto.Message(), )
			pdu = proto.apiMessage.getPDU(msg)
			# get enterprise
			b["enterprise"] = pdu.getComponentByPosition(0)._value
			# get agent address (only gods know why it's here)
			b["agent"] = pdu.getComponentByPosition(1).getComponentByPosition(0).prettyPrint()
			# get generic trap type
			b["trap"] = snmp1traps[pdu.getComponentByPosition(2)._value]
			# get specific trap type
			b["specific"] = pdu.getComponentByPosition(3)._value
			# get timestamp
			b["timestamp"] = pdu.getComponentByPosition(4)._value
			# get variable bindings
			var = pdu.getComponentByPosition(5)
			for i in xrange(len(var)):
				oid = var.getComponentByPosition(i).getComponentByPosition(0)
				val = var.getComponentByPosition(i).getComponentByPosition(1)
				b[oid._value] = self.chain(val)

		elif version == 1:
			b["version"] = "2c"
			proto = api.protoModules[version]
			msg, x = decoder.decode(msg, asn1Spec=proto.Message(), )
			pdu = proto.apiMessage.getPDU(msg)
			var = proto.apiPDU.getVarBindList(pdu)
			b["community"] = msg.getComponentByName("community")._value
			for oid, val in var:
				b[oid._value] = self.chain(val)
		else:
			b["version"] = str(version)
			b["comment"] = "unsupported version"

		self.signal("snmp",b)

	def chain(self,val):
		c = []
		for i in xrange(len(val._componentValues)):
			k = val._componentValues[i]
			if k is not None:
				if hasattr(k,"getComponentType"):
					c.append(k.getComponentType().getNameByPosition(i))
					c.extend(self.chain(k))
				elif hasattr(k,"_value"):
					c.append(k._value)
		return c

class snmp (command):
	"""
	SNMP trap daemon
	"""
	mask = flags.Esoteric

	parameters = {
		"$1": {
			"type": "word",
			"name": "bind",
			"description": "An address to bind to (default: all interfaces)",
			"default": "",
		},
		"port": {
			"type": "int",
			"description": "A UDP port to bind to (default: 1162)",
			"default": 1162,
		},
	}

	def init(self,opts):
		if not opts.call_mask & cmask.Immediate:
			self.s = snmp_socket(self.signal,opts.envl.bind,int(opts.envl.port))
			opts.esoteric.bus.put("monitor",("register",self.s.fd,self.s.read))

	def postUp(self,opts):
		pass
